;;; ===========================================================================
;;; ===========================================================================
;;; == CLASS:     RACECONDITIONS
;;; == FILE NAME: race_conditions.clp
;;; ==
;;; == SPECS:     This file contains the definitions for the "raceconditions" 
;;; ==            class.
;;; ==            The following slots (attributes) are defined for the class
;;; ==            and thus for each raceconditions instance:
;;; ==
;;; ==            - Temperature of track
;;; ==            - Speed of wind on track
;;; ==            - Humidy level at the track
;;; ==            - Air temperature at the track
;;; ==            - Probability of rain
;;; ==            - Weather forecast
;;; ==            - Inches of rainfall
;;; ==            - Best lap time on the track
;;; ==
;;; ==            The following message handlers are defined:
;;; ==
;;; ==            - SetTrackTemperature: Sets the track temperature
;;; ==            - SetWindSpeed: Sets the wind speed
;;; ==            - SetHumidity: Sets the humidity level
;;; ==            - SetAirTemperature: Sets the air temperature
;;; ==            - SetRainProbability: Sets the probability of rain
;;; ==            - SetOutlook: Sets the weather forecast
;;; ==            - SetRainfall: Sets the amount of rainfall on the track
;;; ==            - SetBestOverallLapTime: Sets the best lap time on track
;;; ==
;;; ===========================================================================
;;; ===========================================================================

;;; ===========================================================================
;;; ==                          CLASS DEFINITION
;;; ===========================================================================

(defclass RACECONDITIONS
	(is-a USER)
	(role concrete)

;;; ===========================================================================
;;; ==                               SLOTS
;;; ===========================================================================

	(slot rc_track-temperature
		(type FLOAT))

	(slot rc_wind-speed
		(type FLOAT))

	(slot rc_humidity
		(type FLOAT)
		(range 0.0 100.0))
	
	(slot rc_air-temperature
		(type FLOAT))

	(slot rc_rain-probability
		(type FLOAT)
		(range 0.0 1.0))

	(slot rc_outlook
		(type SYMBOL)
		(allowed-symbols sunny cloudy rainy))

	(slot rc_rainfall
		(type FLOAT))

	(slot rc_best-overall-lap-time
		(type FLOAT))
)

;;; ===========================================================================
;;; ==                         MESSAGE HANDLERS
;;; ===========================================================================
;;; == Weather Conditions
;;; ===========================================================================

(defmessage-handler RACECONDITIONS SetTrackTemperature ()
	(printout t "Enter Track Temperature [F]: ") 
	(bind ?arg (read))
	(if (not (numberp ?arg))
		then 
			(printout t "   -- Please enter a valid value. " ?arg " is NOT a valid value." crlf)
			(send ?self SetTrackTemperature)
		else
			(send ?self put-rc_track-temperature ?arg)
			(printout t "")
	)
)

(defmessage-handler RACECONDITIONS SetWindSpeed ()
	(printout t "Enter Wind Speed [mph]: ") 
	(bind ?arg (read))
	(if (not (numberp ?arg))
		then 
			(printout t "   -- Please enter a valid value. " ?arg " is NOT a valid value." crlf)
			(send ?self SetWindSpeed)
		else
			(send ?self put-rc_wind-speed ?arg)
			(printout t "")
	)
)
(defmessage-handler RACECONDITIONS SetHumidity ()
	(printout t "Enter Humidity Level [0.0 - 100.0]: ") 
	(bind ?arg (read))
	(if (or (not (numberp ?arg)) (or (> ?arg 100.0) (< ?arg 0.0) ) )
		then 
			(printout t "   -- Please enter a valid value. " ?arg " is NOT a valid value." crlf)
			(send ?self SetHumidity)
		else
			(send ?self put-rc_humidity ?arg)
			(printout t "")
	)
)
(defmessage-handler RACECONDITIONS SetAirTemperature ()
	(printout t "Enter the Air Temperature [F]: ") 
	(bind ?arg (read))
	(if (not (numberp ?arg))
		then 
			(printout t "   -- Please enter a valid value. " ?arg " is NOT a valid value." crlf)
			(send ?self SetAirTemperature)
		else
			(send ?self put-rc_air-temperature ?arg)
			(printout t "")
	)
)

(defmessage-handler RACECONDITIONS SetRainProbability ()
	(printout t "Enter Rain Probability [0.0 - 1.0]: ") 
	(bind ?arg (read))
	(if (or (not (numberp ?arg)) (or (> ?arg 1.0) (< ?arg 0.0) ) )
		then 
			(printout t "   -- Please enter a valid value. " ?arg " is NOT a valid value." crlf)
			(send ?self SetRainProbability)
		else
			(send ?self put-rc_rain-probability ?arg)
			(printout t "")
	)
)
	
(defmessage-handler RACECONDITIONS SetOutlook ()
	(printout t "What is the weather outlook? [sunny,cloudy,rainy]: ") 
	(bind ?arg (read))
	(if (eq (member ?arg (slot-allowed-values RACECONDITIONS rc_outlook)) FALSE)
		then 
			(printout t "   -- Please enter a valid value. " ?arg " is NOT a valid value." crlf)
			(send ?self SetOutlook)
		else
			(send ?self put-rc_outlook ?arg)
			(printout t "")
	)
)

(defmessage-handler RACECONDITIONS SetRainfall ()
	(printout t "How many inches of rain are there on the track? : ") 
	(bind ?arg (read))
	(if (not (numberp ?arg))
		then 
			(printout t "   -- Please enter a valid value. " ?arg " is NOT a valid value." crlf)
			(send ?self SetRainfall)
		else
			(send ?self put-rc_rainfall ?arg)
			(printout t "")
	)
)

;;; ===========================================================================
;;; == Race conditions
;;; ===========================================================================

(defmessage-handler RACECONDITIONS SetBestOverallLapTime ()
	(printout t "Enter best overall lap time [minutes]: ") 
	(bind ?arg (read))
	(if (not (numberp ?arg))
		then 
			(printout t "   -- Please enter a valid value. " ?arg " is NOT a valid value." crlf)
			(send ?self SetBestOverallLapTime)
		else
			(send ?self put-rc_best-overall-lap-time ?arg)
			(printout t "")
	)
)

;;; ===========================================================================
;;; ===========================================================================
;;; ==                     END OF RACECONDITIONS CLASS
;;; ===========================================================================
;;; ===========================================================================