;;; ===========================================================================
;;; ===========================================================================
;;; == EXPERT SYSTEM RULES
;;; == FILE NAME: rules.clp
;;; ==
;;; == SPECS:     This file contains the definitions  and declarations for the
;;; ==            rules (knowledge base), and generic functions to be used by
;;; ==            our expert system.
;;; ==
;;; ===========================================================================
;;; ===========================================================================

;;; ===========================================================================
;;; ==                              FUNCTIONS
;;; ===========================================================================
;;; == FUNCTION NAME: yes-or-no-p
;;; == SPEC:          Helper function to take care of yes/no answers given to
;;; ==                the expert system.
;;; ===========================================================================

(deffunction yes-or-no-p (?question)
	(printout t ?question)
	(bind ?response (read))
	(if (not (member$ ?response (create$ yes y no n)))
		then
			(printout t "   -- Please enter (y/n). " ?response " is NOT a valid option." crlf)
			(yes-or-no-p ?question)
		else
			(if (or (eq ?response yes) (eq ?response y))
				then
					TRUE
				else
					FALSE
			)
	)
 )

;;; ===========================================================================
;;; ===========================================================================
;;; ==                           KNOWLEDGE BASE
;;; ===========================================================================
;;; ===========================================================================

;;; ===========================================================================
;;; == Rule checks if current lap time is withing 5 seconds of the best
;;; == overall lap time. If so, driver should stay on track.
;;; ===========================================================================

(defrule kb_rule1
	(declare (salience -100))
	?ins <-(object(is-a RACECAR))
	(object(is-a DRIVER) (driver_current-lap-time ?laptime))
	(object(is-a RACECONDITIONS) (rc_best-overall-lap-time ?bestlaptime))
=>
	(if  (< (- ?bestlaptime ?laptime) 0.05 ) 
		then
			(send ?ins put-racecar_laps-to-pit 0.0)
	)
)

;;; ===========================================================================
;;; == Rule automatically calculates number of laps driver has ramaining
;;; == before going to pits. This is based on the current track length.
;;; ===========================================================================

(defrule kb_rule2
	(declare (salience -101))
	?ins <-(object(is-a RACECAR) (racecar_engine-fuel-load ?fuel) (racecar_engine-fuel-efficiency ?effc))
	(object(is-a TRACK) (track_length ?length))
=>
			(send ?ins put-racecar_laps-to-pit (- (/ (* ?fuel ?effc) ?length) 1))
)

;;; ===========================================================================
;;; == Rule checks if there is any damage done to nose chassis of car.
;;; == If so, it checks whether the lap time is 5 seconds slower than 
;;; == the best overall time. If so, it then instructs the driver to go to pits 
;;; == in the next lap.
;;; ===========================================================================

(defrule kb_rule3
	(declare (salience -102))
	?ins <-(object(is-a RACECAR) (racecar_chassis-nose-damage-level ?damage))
	(object(is-a DRIVER) (driver_current-lap-time ?laptime))
	(object(is-a RACECONDITIONS) (rc_best-overall-lap-time ?bestlaptime))
=>
	(if  (and (> ?damage 2.0) (> (- ?bestlaptime ?laptime) 0.05 ))
		then
			(send ?ins put-racecar_laps-to-pit 1.0)
	)
)

;;; ===========================================================================
;;; == Rule checks if level of rainfall on track is higher than half an inch.
;;; == If so, it checks if current tires are for wet conditions.
;;; == If not, it then instructs the driver to go to pits in the next lap.
;;; ===========================================================================

(defrule kb_rule4
	(declare (salience -103))
	?ins <-(object(is-a RACECAR) (racecar_left-front-tire ?tire)  )
	(object(is-a RACECONDITIONS) (rc_rainfall ?rainfall))
=>
	(if (and (> ?rainfall 0.5) (neq (send ?tire get-tire_type) wet) )
		then
			(send ?ins put-racecar_laps-to-pit 1.0)
	)
)

;;; ===========================================================================
;;; == Rule calculates how much fuel needs to be pumped in the next pit
;;; == stop based on the current lap, current fuel load, and fuel efficiency
;;; ===========================================================================

(defrule kb_rule5
	(declare (salience -104))
	?ins <-(object(is-a RACECAR) (racecar_engine-fuel-efficiency ?effc) (racecar_engine-fuel-capacity ?capacity) (racecar_engine-fuel-load ?fuelload))
	(object(is-a TRACK) (track_laps ?totallaps) (track_length ?length))
	(object(is-a DRIVER) (driver_lap-count ?currentlap))
=>
	(if
		(>= (- ?fuelload (/ (* (- ?totallaps ?currentlap) ?length) ?effc)) 0.0)
			then
				(send ?ins put-racecar_engine-fuel-to-pump 0.0)
			else
				(if 
					(> (* -1.0 (- ?fuelload (/ (* (- ?totallaps ?currentlap) ?length) ?effc))) ?capacity)
						then
							(send ?ins put-racecar_engine-fuel-to-pump (- ?capacity ?fuelload))
						else
							(send ?ins put-racecar_engine-fuel-to-pump (- (* -1.0 (- ?fuelload (/ (* (- ?totallaps ?currentlap) ?length) ?effc))) ?fuelload))
				)
	)
)

;;; ===========================================================================
;;; Rule checks if it is windy and modifies the downforce settings
;;; if lap times are not competitive enough.
;;; ===========================================================================

(defrule kb_rule6
	(declare (salience -105))
	?ins <-(object(is-a RACECAR))
	(object(is-a RACECONDITIONS) (rc_wind-speed ?wind))
=>
	(if (> ?wind 20.0)
		then
			(send ?ins put-racecar_chassis-front-downforce high)
			(send ?ins put-racecar_chassis-rear-downforce high)
		else
			(if (< ?wind 2.0)
				then
					(send ?ins put-racecar_chassis-front-downforce low)
					(send ?ins put-racecar_chassis-rear-downforce low)
				else
					(send ?ins put-racecar_chassis-front-downforce normal)
					(send ?ins put-racecar_chassis-rear-downforce normal)
			)
	)
)

;;; ===========================================================================
;;; Rule checks if the driver should use intermediate tires based
;;; on the rain probability and outlook.
;;; ===========================================================================

(defrule kb_rule7
	(declare (salience -106))
	?ins <-(object(is-a RACECAR))
	(object(is-a RACECONDITIONS) (rc_rain-probability ?prob) (rc_outlook ?outlook))
=>
	(if (and (> ?prob 0.7) (neq ?outlook sunny))
		then
			(send (send ?ins get-racecar_left-front-tire) put-tire_type intermediate )
			(send (send ?ins get-racecar_right-front-tire) put-tire_type intermediate )
			(send (send ?ins get-racecar_left-rear-tire) put-tire_type intermediate )
			(send (send ?ins get-racecar_right-rear-tire) put-tire_type intermediate )
	)
)

;;; ===========================================================================
;;; == Rule will determine if we need to go to full wet tires
;;; ===========================================================================

(defrule kb_rule8
	(declare (salience -107))
	?ins <-(object(is-a RACECAR))
	(object(is-a RACECONDITIONS) (rc_rainfall ?rainfall))
=>
	(if (> ?rainfall 1.0)
		then
			(send (send ?ins get-racecar_left-front-tire) put-tire_type wet )
			(send (send ?ins get-racecar_right-front-tire) put-tire_type wet )
			(send (send ?ins get-racecar_left-rear-tire) put-tire_type wet )
			(send (send ?ins get-racecar_right-rear-tire) put-tire_type wet )
	)
)

;;; ===========================================================================
;;; == Rule will determine if we can go back to dry tires
;;; ===========================================================================

(defrule kb_rule9
	(declare (salience -108))
	?ins <-(object(is-a RACECAR))
	(object(is-a RACECONDITIONS) (rc_rain-probability ?prob) (rc_outlook ?outlook) (rc_rainfall ?rainfall))
=>
	(if (and (and (< ?rainfall 0.5) (eq ?outlook sunny)) (< ?prob 0.5))
		then
			(send (send ?ins get-racecar_left-front-tire) put-tire_type dry )
			(send (send ?ins get-racecar_right-front-tire) put-tire_type dry )
			(send (send ?ins get-racecar_left-rear-tire) put-tire_type dry )
			(send (send ?ins get-racecar_right-rear-tire) put-tire_type dry )
	)
)

;;; ===========================================================================
;;; == Rule will determine if we need to go to super-soft compound tires
;;; == based on the abrassiveness of the track and how hot the track is.
;;; == Note: super-soft tires are only used during qualifying sessions.
;;; ===========================================================================

(defrule kb_rule10
	(declare (salience -109))
	?ins <-(object(is-a RACECAR))
	(object(is-a TRACK) (track_surface-type ?surface) (track_session ?session))
	(object(is-a RACECONDITIONS) (rc_track-temperature ?temperature))
	(test (neq ?session Race))
=>
	(if (and (eq ?surface NonAbrasive) (< ?temperature 65.0 ))
		then
			(send (send ?ins get-racecar_left-front-tire) put-tire_compound super-soft )
			(send (send ?ins get-racecar_right-front-tire) put-tire_compound super-soft )
			(send (send ?ins get-racecar_left-rear-tire) put-tire_compound super-soft )
			(send (send ?ins get-racecar_right-rear-tire) put-tire_compound super-soft )
	)
)

;;; ===========================================================================
;;; == Rule will determine if we need to go to soft compound tires
;;; == based on the abrassiveness of the track and how hot the track is.
;;; ===========================================================================

(defrule kb_rule11
	(declare (salience -110))
	?ins <-(object(is-a RACECAR))
	(object(is-a TRACK) (track_surface-type ?surface))
	(object(is-a RACECONDITIONS) (rc_track-temperature ?temperature))
=>
	(if (and (neq ?surface Abrasive) (< ?temperature 70.0 ))
		then
			(send (send ?ins get-racecar_left-front-tire) put-tire_compound soft )
			(send (send ?ins get-racecar_right-front-tire) put-tire_compound soft )
			(send (send ?ins get-racecar_left-rear-tire) put-tire_compound soft )
			(send (send ?ins get-racecar_right-rear-tire) put-tire_compound soft )
	)
)

;;; ===========================================================================
;;; == Rule will determine if we need to go to medium compound tires
;;; == based on the abrassiveness of the track, the passing dificulty of 
;;; == the track and the passing ability of the driver and aggressiveness.
;;; ===========================================================================

(defrule kb_rule12
	(declare (salience -111))
	?ins <-(object(is-a RACECAR))
	(object(is-a TRACK) (track_surface-type ?surface) (track_passing-dificulty ?passingdiff))
	(object(is-a DRIVER) (driver_passing-skills ?skills) (driver_aggressiveness ?aggressive))
=>
	(if (and (and (and (neq ?surface Abrasive) (< ?passingdiff 3)) (> ?skills 3)) (> ?aggressive 2))
		then
			(send (send ?ins get-racecar_left-front-tire) put-tire_compound medium )
			(send (send ?ins get-racecar_right-front-tire) put-tire_compound medium )
			(send (send ?ins get-racecar_left-rear-tire) put-tire_compound medium )
			(send (send ?ins get-racecar_right-rear-tire) put-tire_compound medium )
	)
)

;;; ===========================================================================
;;; == Rule will determine if we need to go to hard compound tires
;;; == based on the abrassiveness of the track, the passing dificulty of the 
;;; == track.
;;; ===========================================================================

(defrule kb_rule13
	(declare (salience -112))
	?ins <-(object(is-a RACECAR))
	(object(is-a TRACK) (track_surface-type ?surface) (track_passing-dificulty ?passingdiff))
=>
	(if (and (eq ?surface Abrasive) (> ?passingdiff 3)) 
		then
			(send (send ?ins get-racecar_left-front-tire) put-tire_compound hard )
			(send (send ?ins get-racecar_right-front-tire) put-tire_compound hard )
			(send (send ?ins get-racecar_left-rear-tire) put-tire_compound hard )
			(send (send ?ins get-racecar_right-rear-tire) put-tire_compound hard )
	)
)

;;; ===========================================================================
;;; == Rule will determine if we need to go to super-hard compound tires
;;; == based on the abrassiveness of the track, the passing dificulty of the 
;;; == track, and direction of the track.
;;; ===========================================================================

(defrule kb_rule14
	(declare (salience -113))
	?ins <-(object(is-a RACECAR))
	(object(is-a TRACK) (track_surface-type ?surface) (track_passing-dificulty ?passingdiff) (track_race-direction ?direction))
=>
	(if (and (and (eq ?surface Abrasive) (> ?passingdiff 2)) (eq ?direction counterclockwise))
		then
			(send (send ?ins get-racecar_left-front-tire) put-tire_compound super-hard )
			(send (send ?ins get-racecar_right-front-tire) put-tire_compound super-hard )
			(send (send ?ins get-racecar_left-rear-tire) put-tire_compound super-hard )
			(send (send ?ins get-racecar_right-rear-tire) put-tire_compound super-hard )
	)
)

;;; ===========================================================================
;;; == Rule will determine what tire pressure for Monaco based on
;;; == the temperature of the track and the air temperature.
;;; ===========================================================================

(defrule kb_rule15
	(declare (salience -114))
	?ins <-(object(is-a RACECAR))
	(object(is-a TRACK) (track_name ?name))
	(object(is-a RACECONDITIONS) (rc_track-temperature ?tracktemp) (rc_air-temperature ?airtemp))
=>
	(if (eq ?name Monaco)
		then
		(if (and (< ?tracktemp 50.0) (< ?airtemp 60.0))
			then
				(send (send ?ins get-racecar_left-front-tire) put-tire_pressure 40.0 )
				(send (send ?ins get-racecar_right-front-tire) put-tire_pressure 35.0 )
				(send (send ?ins get-racecar_left-rear-tire) put-tire_pressure 40.0 )
				(send (send ?ins get-racecar_right-rear-tire) put-tire_pressure 35.0 )
			else
				(send (send ?ins get-racecar_left-front-tire) put-tire_pressure 35.0 )
				(send (send ?ins get-racecar_right-front-tire) put-tire_pressure 30.0 )
				(send (send ?ins get-racecar_left-rear-tire) put-tire_pressure 35.0 )
				(send (send ?ins get-racecar_right-rear-tire) put-tire_pressure 30.0 )
		)
	)
)

;;; ===========================================================================
;;; == Rule will determine how much pressure to put in tires while on the 
;;; == Brazil track based on the temperature of the track and the 
;;; == air temperature.
;;; ===========================================================================

(defrule kb_rule16
	(declare (salience -115))
	?ins <-(object(is-a RACECAR))
	(object(is-a TRACK) (track_name ?name))
	(object(is-a RACECONDITIONS) (rc_track-temperature ?tracktemp) (rc_air-temperature ?airtemp))
=>
	(if (eq ?name Brazil)
		then
		(if (and (< ?tracktemp 70.0) (< ?airtemp 80.0))
			then
				(send (send ?ins get-racecar_left-front-tire) put-tire_pressure 29.0 )
				(send (send ?ins get-racecar_right-front-tire) put-tire_pressure 35.0 )
				(send (send ?ins get-racecar_left-rear-tire) put-tire_pressure 29.0 )
				(send (send ?ins get-racecar_right-rear-tire) put-tire_pressure 35.0 )
			else
				(send (send ?ins get-racecar_left-front-tire) put-tire_pressure 33.0 )
				(send (send ?ins get-racecar_right-front-tire) put-tire_pressure 39.0 )
				(send (send ?ins get-racecar_left-rear-tire) put-tire_pressure 33.0 )
				(send (send ?ins get-racecar_right-rear-tire) put-tire_pressure 39.0 )
		)
	)
)

;;; ===========================================================================
;;; == Rule will determine how much pressure to put in tireswhile on the 
;;; == Malaysia track based on the temperature of the track and the 
;;; == air temperature.
;;; ===========================================================================

(defrule kb_rule17
	(declare (salience -116))
	?ins <-(object(is-a RACECAR))
	(object(is-a TRACK) (track_name ?name))
	(object(is-a RACECONDITIONS) (rc_track-temperature ?tracktemp) (rc_air-temperature ?airtemp))
=>
	(if (eq ?name Malaysia)
		then
		(if (and (< ?tracktemp 90.0) (< ?airtemp 100.0))
			then
				(send (send ?ins get-racecar_left-front-tire) put-tire_pressure 30.0 )
				(send (send ?ins get-racecar_right-front-tire) put-tire_pressure 25.0 )
				(send (send ?ins get-racecar_left-rear-tire) put-tire_pressure 30.0 )
				(send (send ?ins get-racecar_right-rear-tire) put-tire_pressure 25.0 )
			else
				(send (send ?ins get-racecar_left-front-tire) put-tire_pressure 25.0 )
				(send (send ?ins get-racecar_right-front-tire) put-tire_pressure 22.0 )
				(send (send ?ins get-racecar_left-rear-tire) put-tire_pressure 25.0 )
				(send (send ?ins get-racecar_right-rear-tire) put-tire_pressure 22.0 )
		)
	)
)

;;; ===========================================================================
;;; == Rule will determine how much pressure to put in tires for all other 
;;; == tracks that are clockwise based on the temperature of the track and the 
;;; == air temperature.
;;; ===========================================================================

(defrule kb_rule18
	(declare (salience -117))
	?ins <-(object(is-a RACECAR))
	(object(is-a TRACK) (track_name ?name) (track_race-direction ?direction) )
	(object(is-a RACECONDITIONS) (rc_track-temperature ?tracktemp) (rc_air-temperature ?airtemp))
=>
	(if (and (not (member$ ?name (create$ Monaco Brazil Malaysia))) (eq ?direction clockwise) )
		then
		(if (and (< ?tracktemp 60.0) (< ?airtemp 70.0))
			then
				(send (send ?ins get-racecar_left-front-tire) put-tire_pressure 30.0 )
				(send (send ?ins get-racecar_right-front-tire) put-tire_pressure 25.0 )
				(send (send ?ins get-racecar_left-rear-tire) put-tire_pressure 30.0 )
				(send (send ?ins get-racecar_right-rear-tire) put-tire_pressure 25.0 )
			else
				(send (send ?ins get-racecar_left-front-tire) put-tire_pressure 28.0 )
				(send (send ?ins get-racecar_right-front-tire) put-tire_pressure 25.0 )
				(send (send ?ins get-racecar_left-rear-tire) put-tire_pressure 28.0 )
				(send (send ?ins get-racecar_right-rear-tire) put-tire_pressure 25.0 )
		)
	)
)

;;; ===========================================================================
;;; == Rule will determine what tire pressure setting to use for all other 
;;; == tracks that are counterclockwise based on the temperature of the track 
;;; == and the air temperature.
;;; ===========================================================================

(defrule kb_rule19
	(declare (salience -118))
	?ins <-(object(is-a RACECAR))
	(object(is-a TRACK) (track_name ?name) (track_race-direction ?direction) )
	(object(is-a RACECONDITIONS) (rc_track-temperature ?tracktemp) (rc_air-temperature ?airtemp))
=>
	(if (and (not (member$ ?name (create$ Monaco Brazil Malaysia))) (eq ?direction counterclockwise) )
		then
		(if (and (< ?tracktemp 60.0) (< ?airtemp 70.0))
			then
				(send (send ?ins get-racecar_left-front-tire) put-tire_pressure 25.0 )
				(send (send ?ins get-racecar_right-front-tire) put-tire_pressure 30.0 )
				(send (send ?ins get-racecar_left-rear-tire) put-tire_pressure 25.0 )
				(send (send ?ins get-racecar_right-rear-tire) put-tire_pressure 30.0 )
			else
				(send (send ?ins get-racecar_left-front-tire) put-tire_pressure 25.0 )
				(send (send ?ins get-racecar_right-front-tire) put-tire_pressure 28.0 )
				(send (send ?ins get-racecar_left-rear-tire) put-tire_pressure 25.0 )
				(send (send ?ins get-racecar_right-rear-tire) put-tire_pressure 28.0 )
		)
	)
)

;;; ===========================================================================
;;; == Rule will adjust the tire pressure for late breaking style drivers
;;; == for clockwise tracks.
;;; ===========================================================================

(defrule kb_rule20
	(declare (salience -119))
	?ins <-(object(is-a RACECAR))
	(object(is-a DRIVER) (driver_breaking-style ?style))
	(object(is-a TRACK) (track_race-direction ?direction))
	(test (eq ?direction clockwise))
	(test (eq ?style late))
=>
	(send (send ?ins get-racecar_left-front-tire) put-tire_pressure (+ (send (send ?ins get-racecar_left-front-tire) get-tire_pressure) 2.0))
)

;;; ===========================================================================
;;; == Rule will adjust the tire pressure for late breaking style drivers
;;; == for counterclockwise tracks.
;;; ===========================================================================

(defrule kb_rule21
	(declare (salience -120))
	?ins <-(object(is-a RACECAR))
	(object(is-a DRIVER) (driver_breaking-style ?style))
	(object(is-a TRACK) (track_race-direction ?direction))
	(test (eq ?direction counterclockwise))
	(test (eq ?style late))
=>
	(send (send ?ins get-racecar_left-front-tire) put-tire_pressure (+ (send (send ?ins get-racecar_right-front-tire) get-tire_pressure) 2.0))
)

;;; ===========================================================================
;;; == Rule will adjust the tire pressure for early breaking style drivers
;;; == for clockwise tracks.
;;; ===========================================================================

(defrule kb_rule22
	(declare (salience -121))
	?ins <-(object(is-a RACECAR))
	(object(is-a DRIVER) (driver_breaking-style ?style))
	(object(is-a TRACK) (track_race-direction ?direction))
	(test (eq ?direction clockwise))
	(test (eq ?style early))
=>
	(send (send ?ins get-racecar_left-front-tire) put-tire_pressure (- (send (send ?ins get-racecar_left-front-tire) get-tire_pressure) 1.0))
)

;;; ===========================================================================
;;; == Rule will adjust the tire pressure for early breaking style drivers
;;; == for counterclockwise tracks.
;;; ===========================================================================

(defrule kb_rule23
	(declare (salience -122))
	?ins <-(object(is-a RACECAR))
	(object(is-a DRIVER) (driver_breaking-style ?style))
	(object(is-a TRACK) (track_race-direction ?direction))
	(test (eq ?direction counterclockwise))
	(test (eq ?style early))
=>
	(send (send ?ins get-racecar_left-front-tire) put-tire_pressure (- (send (send ?ins get-racecar_right-front-tire) get-tire_pressure) 1.5))
)

;;; ===========================================================================
;;; == Rule will adjust the tire pressure for normal breaking style drivers
;;; == for clockwise tracks.
;;; ===========================================================================

(defrule kb_rule24
	(declare (salience -123))
	?ins <-(object(is-a RACECAR))
	(object(is-a DRIVER) (driver_breaking-style ?style))
	(object(is-a TRACK) (track_race-direction ?direction))
	(test (eq ?direction clockwise))
	(test (eq ?style normal))
=>
	(send (send ?ins get-racecar_left-front-tire) put-tire_pressure (- (send (send ?ins get-racecar_left-front-tire) get-tire_pressure) 0.5))
)

;;; ===========================================================================
;;; == Rule will adjust the tire pressure for normal breaking style drivers
;;; == for counterclockwise tracks.
;;; ===========================================================================

(defrule kb_rule25
	(declare (salience -124))
	?ins <-(object(is-a RACECAR))
	(object(is-a DRIVER) (driver_breaking-style ?style))
	(object(is-a TRACK) (track_race-direction ?direction))
	(test (eq ?direction counterclockwise))
	(test (eq ?style normal))
=>
	(send (send ?ins get-racecar_left-front-tire) put-tire_pressure (+ (send (send ?ins get-racecar_right-front-tire) get-tire_pressure) 0.5))
)

;;; ===========================================================================
;;; ===========================================================================
;;; ==                         GENERAL QUERY RULES
;;; ===========================================================================
;;; ===========================================================================

;;; ===========================================================================
;;; == Checks if user wants to continue running the expert system
;;; ===========================================================================

(defrule ask-continue
	(declare (salience 0))
	(not (CONTINUE))
=>
	(if (yes-or-no-p "Do you want to continue running the F1 expert system? (y/n): ")
		then
			(assert (CONTINUE))
			(printout t crlf crlf)
	)
)

;;; ===========================================================================
;;; ===========================================================================
;;; ==                            TRACK QUESTIONS 
;;; ===========================================================================
;;; ===========================================================================

;;; ===========================================================================
;;; == If the user decides to continue, ask if he/she wants to change 
;;; == track settings.
;;; ===========================================================================

(defrule ask-change-track-settings
	(declare (salience -1))
	(CONTINUE)
	(not (FIRSTRUN))
=>
	(if (yes-or-no-p "Do you want to change the track settings? (y/n): ")
		then
			(assert (CHANGE_TRACK))
			(printout t crlf)
	)
)

(defrule enter-track-settings
	(declare (salience -2))
	(or (CHANGE_TRACK) (FIRSTRUN))
=>
	(printout t crlf)
	(printout t "**************************" crlf)
	(printout t "**     TRACK SETTINGS   **" crlf)
	(printout t "**************************" crlf)
	
	(send [track] SetTrack)
	(send [track] SetSession)
	(send [raceconditions] SetBestOverallLapTime)

	(printout t "**************************" crlf crlf)
)

;;; ===========================================================================
;;; ===========================================================================
;;; ==                            RACE QUESTIONS
;;; ===========================================================================
;;; ===========================================================================

;;; ===========================================================================
;;; == If the user decides to continue, ask if he/she wants to change 
;;; == weather conditions.
;;; ===========================================================================

(defrule ask-change-weather-conditions
	(declare (salience -3))
	(CONTINUE)
	(not (FIRSTRUN))
=>
	(if (yes-or-no-p "Do you want to change the weather conditions? (y/n): ")
		then
			(assert (CHANGE_WEATHER_CONDITIONS))
			(printout t crlf)
	)
)


(defrule enter-weather-conditions
	(declare (salience -4))
	(or (CHANGE_WEATHER_CONDITIONS) (FIRSTRUN))
=>
	(printout t crlf)
	(printout t "**************************" crlf)
	(printout t "**  WEATHER CONDITIONS  **" crlf)
	(printout t "**************************" crlf)
	
	(send [raceconditions] SetTrackTemperature)
	(send [raceconditions] SetWindSpeed)
	(send [raceconditions] SetHumidity)
	(send [raceconditions] SetAirTemperature)
	(send [raceconditions] SetRainProbability)
	(send [raceconditions] SetOutlook)
	;;; Only ask for rain inches if the humidity levels and rain probability is high enough
	(if (and (eq (send [raceconditions] get-rc_outlook) rainy) (and (> (send [raceconditions] get-rc_rain-probability) 0.70) (> (send [raceconditions] get-rc_humidity) 60.0) ))
		then
			(send [raceconditions] SetRainfall)
		else
			(send [raceconditions] put-rc_rainfall 0.0)
	)
	
	
	(printout t "**************************" crlf crlf)
)


;;; ===========================================================================
;;; ===========================================================================
;;; ==                            DRIVER STATUS
;;; ===========================================================================
;;; ===========================================================================

;;; ===========================================================================
;;; == If the user decides to continue, ask if he/she wants to change 
;;; == driver status.
;;; ===========================================================================

(defrule ask-change-driver-status
	(declare (salience -5))
	(CONTINUE)
	(not (FIRSTRUN))
=>
	(if (yes-or-no-p "Do you want to change the Driver's status? (y/n): ")
		then
			(assert (CHANGE_DRIVER_STATUS))
			(printout t crlf)
	)
)


(defrule enter-driver-status
	(declare (salience -6))
	(or (CHANGE_DRIVER_STATUS) (FIRSTRUN))
=>
	(printout t crlf)
	(printout t "**************************" crlf)
	(printout t "**     DRIVER STATUS    **" crlf)
	(printout t "**************************" crlf)
	
	(send [driver] SetDriverLapCount (send [track] get-track_laps))
	(send [driver] SetDriverCurrentLapTime)
	(send [driver] SetDriverBrakingStyle)
	(send [racecar] SetRacecarFuelLoad)
	(send [racecar] SetRacecarChassisNoseDamageLevel)
	
	(printout t "**************************" crlf crlf)
 
)


;;; ===========================================================================
;;; ===========================================================================
;;; ==                           PRINT RESULTS
;;; ===========================================================================
;;; ===========================================================================

(defrule show-results
	(declare (salience -1000))
	(CONTINUE)
=>
	(assert (SHOWRESULTS))
)

(defrule print-results
	(SHOWRESULTS)
=>
	(printout t crlf crlf)
	(printout t "*************************************************" crlf)
	(printout t "**                    RESULTS                  **" crlf)
	(printout t "*************************************************" crlf crlf)
	(printout t "Number of laps before going to pits: ") 
	(if (<= (send [racecar] get-racecar_laps-to-pit) 1.0)
		then
			(printout t "YOU MUST GO TO PITS NOW!!" crlf)
		else
			(printout t (send [racecar] get-racecar_laps-to-pit) crlf)
	)
	(printout t "Gallons of fuel to pump during pit stop: ")
	(if (> (send [racecar] get-racecar_engine-fuel-to-pump) 0.0)
		then
			(printout t (send [racecar] get-racecar_engine-fuel-to-pump) crlf)
		else
			(printout t "No need to refuel." crlf)
	)
	(printout t "Front Downforce: " (send [racecar] get-racecar_chassis-front-downforce) crlf)
	(printout t "Rear Downforce: " (send [racecar] get-racecar_chassis-rear-downforce) crlf)
	(printout t "****************      TIRES   *******************" crlf )
	(printout t "****************   LEFT FRONT  ******************"  crlf)
	(printout t "       Type: " (send (send [racecar] get-racecar_left-front-tire) get-tire_type) crlf)
	(printout t "   Compound: " (send (send [racecar] get-racecar_left-front-tire) get-tire_compound) crlf)
	(printout t "   Pressure: " (send (send [racecar] get-racecar_left-front-tire) get-tire_pressure) crlf)
	(printout t "****************   RIGHT FRONT ******************"  crlf)
	(printout t "       Type: " (send (send [racecar] get-racecar_right-front-tire) get-tire_type) crlf)
	(printout t "   Compound: " (send (send [racecar] get-racecar_right-front-tire) get-tire_compound) crlf)
	(printout t "   Pressure: " (send (send [racecar] get-racecar_right-front-tire) get-tire_pressure) crlf)
	(printout t "****************    LEFT REAR  ******************"  crlf)
	(printout t "       Type: " (send (send [racecar] get-racecar_left-rear-tire) get-tire_type) crlf)
	(printout t "   Compound: " (send (send [racecar] get-racecar_left-rear-tire) get-tire_compound) crlf)
	(printout t "   Pressure: " (send (send [racecar] get-racecar_left-rear-tire) get-tire_pressure) crlf)
	(printout t "***************    RIGHT REAR  ******************"  crlf)
	(printout t "       Type: " (send (send [racecar] get-racecar_right-rear-tire) get-tire_type) crlf)
	(printout t "   Compound: " (send (send [racecar] get-racecar_right-rear-tire) get-tire_compound) crlf)
	(printout t "   Pressure: " (send (send [racecar] get-racecar_right-rear-tire) get-tire_pressure) crlf)
	(printout t "*************************************************" crlf)
	(printout t crlf crlf)

	(retract *)
)

;;; ===========================================================================
;;; ===========================================================================
;;; ==                                START-UP
;;; ===========================================================================
;;; ===========================================================================


 (defrule system-banner ""
	  (declare (salience 10))
  =>
	  (printout t crlf crlf)
	  (printout t "*************************************" crlf)
	  (printout t "** Ferrari Formula 1 Expert System **" crlf)
	  (printout t "*************************************" crlf)
	  (assert (CONTINUE))
	  (assert (FIRSTRUN))
  )

;;; ===========================================================================
;;; ===========================================================================
;;; ==                          END OF RULES
;;; ===========================================================================
;;; ===========================================================================	